
# --- the function load the CORDEX data for selected gcm+rcm simulation for the dates of interest
# --- it performs the interpolation to standard calendar in case this condition is not fullfilled by the model 
# --- data for historical and projections are returned as one matrix
# --- input required: 
# --- gcm = GCM simulation 
# --- rcm = RCM simulation
# --- path_in_mod = path to CORDEX data 
# --- variable = tasmin, tasmax or pr
# --- cells = indices of cells in the grid to be processed
# --- dates.historical = daily dates of historical period
# --- dates.projections = daily dates over projection

load_CORDEX<-function(gcm, rcm, path_in_mod, variable, cells, dates.historical, dates.projections ){
    # available simulations for the model i.mod
    n.files<-list.files(paste0(path_in_mod,variable,"/"),pattern=paste0(c(gcm,rcm),collapse="|")) 
    idx.hist<-grep("historical",n.files);idx.proj<-grep(scenario,n.files) 
    
    # final data array with standard calendar
    mat.hist.full<-matrix(nrow=length(dates.historical),ncol=length(cells))
    mat.proj.full<-matrix(nrow=length(dates.projections),ncol=length(cells))
    
    # used for loading only
    dates.hist<-numeric(); dates.proj<-numeric() 
    mat.hist<-matrix(nrow=0,ncol=length(cells)); 
    mat.proj<-matrix(nrow=0,ncol=length(cells))
    
    # load all files for historical period
    for(i.file.hist in seq_along(idx.hist)){
      r<-nc_open(paste0(path_in_mod,variable,"/",n.files[idx.hist[i.file.hist]]))
      values<-(ncvar_get(r,variable)); tmp<-as.vector(values); 
      # load data in a tmp matrix with row=time and col=grid points
      mat.tmp<-t(matrix(tmp,ncol=length(values[1,1,]),nrow=length(values[1,,1])*length(values[,1,1])))
      # select only cells of interest
      mat.tmp<-mat.tmp[,cells]; 
      # append to the full matrix
      mat.hist<-rbind(mat.hist,mat.tmp)
      
      # get time and append to the total vector of dates
      times<-nc.get.time.series(r)
      dates.hist<-append(dates.hist,times,length(dates.hist))
    }
    # truncate hours
    dates.hist<-trunc(dates.hist,"day")
    # remove data outside the considered historical period
    rm<-which(year(dates.hist)<min(year(dates.historical)) | year(dates.hist)>max(year(dates.historical)))
    if(length(rm)>0){ dates.hist<-dates.hist[-rm]; mat.hist<-mat.hist[-rm,]}
    
    # check if time series is of expected length
    if(length(dates.historical)==length(dates.hist)) mat.hist.full<-mat.hist
    # otherwise it is not a standard calendar and interpolation is needed
    if(length(dates.historical)!=length(dates.hist)){
      # days are randomly inserted to reach the expected length of time series
      seq_pcict<-seq(1,length(dates.hist),length.out=length(dates.historical))
      seq_pcict<-round(seq_pcict)
      for(k in 1:length(cells)){
        mat.hist.full[,k]<-mat.hist[seq_pcict,k]
      }
    }
    # start the loading for projections
    for(i.file.proj in seq_along(idx.proj)){
      r<-nc_open(paste0(path_in_mod,variable,"/",n.files[idx.proj[i.file.proj]]))
      values<-(ncvar_get(r,variable)); tmp<-as.vector(values); 
      mat.tmp<-t(matrix(tmp,ncol=length(values[1,1,]),nrow=length(values[1,,1])*length(values[,1,1])))
      mat.tmp<-mat.tmp[,cells]; mat.proj<-rbind(mat.proj,mat.tmp)
      times<-nc.get.time.series(r)
      dates.proj<-append(dates.proj,times,length(dates.proj))
    }
    dates.proj<-trunc(dates.proj,"day")
    rm<-which(year(dates.proj)<min(year(dates.projections)) | year(dates.proj)>max(year(dates.projections)))
    if(length(rm)>0){ dates.proj<-dates.proj[-rm]; dates.proj<-mat.proj[-rm,]}
    if(length(dates.projections)==length(dates.proj)) mat.proj.full<-mat.proj
    if(length(dates.projections)!=length(dates.proj)){
      seq_pcict<-seq(1,length(dates.proj),length.out=length(dates.projections))
      seq_pcict<-round(seq_pcict)
      for(k in 1:length(cells)){
        mat.proj.full[,k]<-mat.proj[seq_pcict,k]
      }
    }
  
  # append time and data for historical and projections and return the model
  mod.dat<-rbind(mat.hist.full,mat.proj.full)
  return(mod.dat)
 }
